@@ -14,10 +14,10 @@
  *
  */
 
-#include "common.h"
+#include "epos.h"
 #include "agent.h"
 #include "client.h"
-
+#include "slab.h"
 
 #ifdef HAVE_SYS_TIME_H
 	#include <sys/time.h>
@@ -62,6 +62,38 @@
 
 #define DARK_ERRLOG 2	/* 2 == stderr; for global stdshriek and stddbg output */
 
+#ifdef HAVE_GETTIMEOFDAY
+
+inline void agent_profile(const char *s)
+{
+	if (!scfg->profile || !*scfg->profile) return;
+	
+	static FILE *log = NULL;
+	if (!log) log = fopen(scfg->profile, "w", "profile");
+	static struct timeval start, stop;
+	if (!s) {
+		if (gettimeofday(&start, NULL)) shriek(861, "profiler fails");
+		int duration = start.tv_sec - stop.tv_sec;
+		duration *= 1000000;
+		duration += start.tv_usec - stop.tv_usec;
+		fprintf(log, "%10ld", duration);
+		fflush(log);
+		if (gettimeofday(&start, NULL)) shriek(861, "profiler fails");
+	} else {
+		if (gettimeofday(&stop, NULL)) shriek(861, "profiler fails");
+		int duration = stop.tv_sec - start.tv_sec;
+		duration *= 1000000;
+		duration += stop.tv_usec - start.tv_usec;
+		fprintf(log, " %-13s%8ld\n", s, duration);
+		fflush(log);
+		if (gettimeofday(&stop, NULL)) shriek(861, "profiler fails");
+	}
+}
+
+#else
+	inline void agent_profile(const char *s) { return; }
+#endif
+
 
 agent::agent(DATA_TYPE typein, DATA_TYPE typeout)
 {
@@ -70,12 +102,12 @@ agent::agent(DATA_TYPE typein, DATA_TYPE typeout)
 	pendin = pendout = NULL; pendcount = 0;
 	c = NULL;
 	dep = NULL;
-	DEBUG(1,11,fprintf(STDDBG, "Creating a handler, intype %i, outtype %i\n", typein, typeout);)
+	D_PRINT(1, "Creating a handler, intype %i, outtype %i\n", typein, typeout);
 }
 
 agent::~agent()
 {
-//	DEBUG(1,11,fprintf(STDDBG, "Handler deleted.\n");)
+//	D_PRINT(1, "Handler deleted.\n");
 }
 
 inline void
@@ -89,42 +121,53 @@ void
 agent::timeslice()
 {
 	c->enter();
-	DEBUG(1,11,fprintf(STDDBG, "Timeslice for %s\n", name());)
-	DEBUG(0,11,fprintf(STDDBG, "pendcount %d, next->pendcount %d, pend_max %d\n", pendcount, next ? next->pendcount : -42, cfg->pend_max);)
-	if (next && next->pendcount > cfg->pend_max) {
-		DEBUG(1,11,fprintf(STDDBG, "(satiated)\n");)
+	D_PRINT(1, "Timeslice for %s\n", name());
+	D_PRINT(0, "pendcount %d, next->pendcount %d, pend_max %d\n", pendcount, next ? next->pendcount : -42, scfg->pend_max);
+	if (next && next->pendcount > scfg->pend_max) {
+		D_PRINT(1, "(satiated)\n");
 		c->leave();
 		return;
 	}
 	if (!inb && in != T_NONE) {
 		if (!pendin) {
 			/* We may take this path legally after an intr command */
-			DEBUG(2,11,fprintf(STDDBG, "No input of type %i; shrugging off\n", in);)
+			D_PRINT(2, "No input of type %i; shrugging off\n", in);
 			c->leave();
 			return;
 		}
-		DEBUG(2,11,fprintf(STDDBG, "Getting pending task, type %i\n", in);)
+		D_PRINT(2, "Getting pending task, type %i\n", in);
 		pend_ll *tmp = pendin;
 		inb = tmp->d;
 		pendin = tmp->next;
 		delete tmp;
-		if (pendcount-- == cfg->pend_min) prev->unquench();
+		if (pendcount-- == scfg->pend_min) prev->unquench();
 		if (!pendin) {
 			prev->pendout = NULL;
 			if (pendcount) shriek(862, "pending count incorrect");
 		}
 	}
 	try {
+		agent_profile(NULL);
 		run();
+		agent_profile(name());
 	} catch (command_failed *e) {
 		if (!next) throw e;
-		DEBUG(2,11,fprintf(STDDBG, "Processing failed, %d, %.60s\n", e->code, e->msg);)
+		D_PRINT(2, "Processing failed, %d, %.60s\n", e->code, e->msg);
 		reply(e->code, e->msg);
 		delete e;
 		finis(true);
 	}
 	if (pendcount && !inb) schedule();
 	c->leave();
+
+	agent *tmp;
+	for (agent *a = dep; a; a = tmp) {
+		D_PRINT(1, "Scheduling %s because of %s\n", a->name(), name());
+		tmp = a->dep;
+		a->schedule();
+		a->dep = NULL;
+	}
+	dep = NULL;
 }
 
 bool
@@ -141,7 +184,8 @@ inline void do_relax(void *ptr, DATA_TYPE type)
 		case T_STML:	
 		case T_TEXT:	free(ptr); break;
 		case T_UNITS:	delete ((unit *)ptr); break;
-		case T_DIPHS:	free(ptr); break;
+		case T_SEGS:	free(ptr); break;
+		case T_SSIF:	free(ptr); break;
 		case T_WAVEFM:	delete((wavefm *)ptr); break;
 	}
 }
@@ -181,10 +225,11 @@ agent::finis(bool err)
  *	relax() would call sync_soundcard() instead.
  */
 
-void agent::brk()
+bool agent::brk()
 {
 	relax();
-	DEBUG(1,11,fprintf(STDDBG, "interrupting an agent, intype %i, outtype %i\n", in, out);)
+	D_PRINT(1, "interrupting an agent, intype %i, outtype %i\n", in, out);
+	return true;
 }
 
 void
@@ -264,9 +309,8 @@ void
 a_print::run()
 {
 	char *b;
-	int l;
 
-	b = ((unit *)inb)->gather(&l, false /* no ^$ */, true /* incl. ssegs */ );
+	b = ((unit *)inb)->gather(false /* no ^$ */, true /* incl. ssegs */ );
 
 	delete (unit *) inb;
 	inb = NULL;
@@ -280,41 +324,78 @@ class a_segs : public agent
 	virtual const char *name() { return "segs"; };
 	int position;
    public:
-	a_segs() : agent(T_UNITS, T_DIPHS) {position = 0;};
+	a_segs() : agent(T_UNITS, T_SEGS) {position = 0;};
 };
 
-#define INIT_DIPHS_BS	2048
+#define INIT_SEGS_BS	2048
+#define INIT_SSIF_BS	4096
 
 void
 a_segs::run()
 {
-	int dbs = cfg->db_size ? cfg->db_size : INIT_DIPHS_BS;
-	segment *d = (segment *)xmalloc((dbs + 1) * sizeof(segment));
+	int sbs = cfg->seg_buff_size ? cfg->seg_buff_size : INIT_SEGS_BS;
+	segment *d = (segment *)xmalloc((sbs + 1) * sizeof(segment));
 	segment *c = d + 1;
 	int n;
 	int items = 0;
 
 	unit *root = *(unit **)&inb;
+	root->project(scfg->_segm_level);
 again:
-	n = root->write_segs(c, position, dbs);
+	n = root->write_segs(c, position, sbs);
 	position += n;
 	items += n;
-	if (cfg->db_size && n >= dbs) {
-		d = (segment *)xrealloc(d, (dbs + 1 + position) * sizeof(segment));
+	if (cfg->seg_buff_size && n >= sbs) {
+		d = (segment *)xrealloc(d, (sbs + 1 + position) * sizeof(segment));
 		c = d + 1 + position;
 		goto again;
-	}
-	if (!cfg->db_size || n < dbs) {
+	} else {
 		delete root;
 		inb = NULL;
 		position = 0;
-	} else schedule();
+	}
 	d->code = items;
 	d->nothing = d->ll = 0;
-	DEBUG(1,11,fprintf(STDDBG, "agent segs generated %d segments\n", n);)
+	D_PRINT(1, "agent segs generated %d segments\n", n);
 	pass(d);
 }
 
+class a_ssif : public agent
+{
+	virtual void run();
+	virtual const char *name() { return "to-mbrola"; };
+	int position;
+   public:
+	a_ssif() : agent(T_UNITS, T_SSIF) {position = 0;};
+};
+
+void
+a_ssif::run()
+{
+	int ssifbs = cfg->ssif_buff_size ? cfg->ssif_buff_size : INIT_SSIF_BS;
+	char *d = (char *)xmalloc((ssifbs + 1));
+	char *c = d /* + sizeof header */;
+	int n;
+	int items = 0;
+
+	unit *root = *(unit **)&inb;
+	root->project(scfg->_phone_level);
+again:
+	n = root->write_ssif(c, position, ssifbs);
+	position += n;
+	items += n;
+	if (cfg->ssif_buff_size && n >= ssifbs) {
+		d = (char *)xrealloc(d, (ssifbs + 1 + position));
+		c = d /* + sizeof header */ + strlen(d);		/* FIXME: efficiency */
+		goto again;
+	} else {
+		delete root;
+		inb = NULL;
+		position = 0;
+	}
+	D_PRINT(1, "agent ssif generated %d items\n", n);
+	pass(d);
+}
 
 class a_chunk : public agent
 {
@@ -328,11 +409,30 @@ class a_chunk : public agent
 inline char *utt_break(char *t)		/* returns ptr past the last char */
 {
 	char *r = t;
-	do {
+	do {				/* split between . ? ... and whitespace */
 		r += strcspn(r, ".?");
 		r += strspn(r, ".?");
 	} while (!strchr(WHITESPACE, r[0]));
-	if (!r[0]) return NULL;
+
+	if (r - t > scfg->max_utterance) {
+		r = t + strcspn(t, ".,?!:;");
+		if (*r) r++;
+	} else return r;
+
+	if (r - t > scfg->max_utterance) {
+		r = t + strcspn(t, ".,?!:;-=+_~@#$%^&*\\|/ \t\n");
+		if (*r) r++;
+	} else return r;
+
+	if (r - t > scfg->max_utterance) {
+		r = t + strcspn(t, "()<>{}[]'\"");
+	} else return r;
+
+	if (r - t > scfg->max_utterance) {
+		if (scfg->split_utterance < (int)strlen(t))
+			r = t + scfg->split_utterance;
+	}
+
 	return r;
 }
 
@@ -343,6 +443,7 @@ void a_chunk::run()
 	}
 	char *tmp = bm;
 	bm = utt_break(bm);
+	D_PRINT(2, "Utterance chunking about to split off %d bytes\n", bm - tmp);
 	if (bm && *bm) {
 		char h = bm[0];
 		bm[0] = 0;
@@ -353,6 +454,7 @@ void a_chunk::run()
 		pass(strdup(tmp));
 		free(inb);
 		inb = NULL;
+		bm = NULL;
 	}
 }
 
@@ -378,7 +480,7 @@ a_join::run()
 		free(heldout); heldout = NULL;
 	} else b = last;
 
-	if (utt_break(b)) {
+	if (*utt_break(b)) {
 		pass(b);
 	} else heldout = b;
 }
@@ -387,39 +489,74 @@ class a_synth : public agent
 {
 	virtual void run();
 	virtual const char *name() { return "synth"; };
+   protected:
+	void init_syn();
    public:
-	a_synth() : agent(T_DIPHS, T_WAVEFM) {};
+	a_synth(DATA_TYPE din, DATA_TYPE dout) : agent(din, dout) {};
+	a_synth() : agent(T_SEGS, T_WAVEFM) {};
 };
 
+bool fallbackable_error(int code)
+{
+	switch (this_lang->fallback_mode) {
+		case 0: return false;
+		case 1: return true;
+		case 4: return code == 445 || code / 10 == 47;
+		case 7: return code / 10 == 47;
+		default: return code == this_lang->fallback_mode;
+	}
+}
+
+void
+a_synth::init_syn()
+{
+	try {
+		this_voice->syn = setup_synth(this_voice);
+	} catch (command_failed *e) {
+		if (fallbackable_error(e->code)
+				&& this_lang->fallback_voice
+				&& *this_lang->fallback_voice) {
+			voice_switch(this_lang->fallback_voice);
+			if (this_lang->permanent_fallbacks) {
+				c->leave();
+				voice_switch(this_lang->fallback_voice);
+				c->enter();
+			} 
+			delete e;
+			return;
+		} else throw e;
+	}
+}
+
 void
 a_synth::run()
 {
 	if (!this_voice) shriek(861, "No current voice");
-	if (!this_voice->syn) {
-		try {
-			synth *sy = setup_synth(this_voice);
-			this_voice->syn = sy;
-//			c->leave();
-//			this_voice->syn = sy;
-//			c->enter();
-		} catch (command_failed *e) {
-			if (e->code / 10 == 47 && this_lang->fallback_voice && *this_lang->fallback_voice) {
-								// 47x codes are network errors
-				voice_switch(this_lang->fallback_voice);
-				if (this_lang->permanent_fallbacks) {
-					c->leave();
-					voice_switch(this_lang->fallback_voice);
-					c->enter();
-				} 
-				delete e;
-				run();
-				return;
-			} else throw e;
-		}
-	}
+	while (!this_voice->syn) init_syn();	// at most twice
 	wavefm *wfm = new wavefm(this_voice);
 	this_voice->syn->synsegs(this_voice, (segment *)inb + 1, ((segment*)inb)->code, wfm);
-	DEBUG(1,11,fprintf(STDDBG, "a_synth processes %d segments\n", ((segment *)inb)->code);)
+	D_PRINT(1, "a_synth processes %d segments\n", ((segment *)inb)->code);
+
+	free(inb);
+	inb = NULL;
+	pass(wfm);
+}
+
+class a_syn : public a_synth
+{
+	virtual void run();
+   public:
+	a_syn() : a_synth(T_SSIF, T_WAVEFM) {};
+};
+
+void
+a_syn::run()
+{
+	if (!this_voice) shriek(861, "No current voice");
+	while (!this_voice->syn) init_syn();	// at most twice
+	wavefm *wfm = new wavefm(this_voice);
+	this_voice->syn->synssif(this_voice, (char *)inb, wfm);
+//	D_PRINT(1, "a_synth processes %d segments\n", ((segment *)inb)->code);
 
 	free(inb);
 	inb = NULL;
@@ -462,17 +599,17 @@ extern int localsound;
 socky int special_io(const char *name, DATA_TYPE intype)
 {
 	if (intype == T_INPUT || strcmp(name, LOCALSOUNDAGENT))
-		shriek(415, fmt("Bad stream component %s", name));
+		shriek(415, "Bad stream component %s", name);
 	if (!cfg->localsound) shriek(453, "Not allowed to use localsound");
 
 	if (localsound != -1) return localsound;
-	int r = open(cfg->local_sound_device, O_WRONLY | O_NONBLOCK);
-	if (r == -1) shriek(462, fmt("Could not open localsound device, error %d", errno));
+	int r = open(scfg->local_sound_device, O_WRONLY | O_NONBLOCK);
+	if (r == -1) shriek(462, "Could not open localsound device, error %d", errno);
 	localsound = r;
 	return r;
 }
 
-void stretch_sleep_table(socky int);
+void stretch_sleep_tables(socky int);
 
 a_io::a_io(const char *par, DATA_TYPE in, DATA_TYPE out) : agent(in, out)
 {
@@ -484,17 +621,17 @@ a_io::a_io(const char *par, DATA_TYPE in, DATA_TYPE out) : agent(in, out)
 	switch(*par) {
 		case '$': dc = data_conns->translate(par + 1);
 			  if (!dc) shriek(444, "Not a known data connection handle");
-			  else socket = (in == T_INPUT ? dc->c->config->sd_in : dc->c->config->sd_out);
+			  else socket = (in == T_INPUT ? dc->c->config->get__sd_in() : dc->c->config->get__sd_out());
 			  break;
 		case '/': if (in == T_INPUT && !cfg->readfs)
-				shriek(451, "No filesystem inputs allowed");
+				shriek(454, "No filesystem inputs allowed");
 			  if (out == T_NONE && !cfg->writefs)
-			  	shriek(451, "No filesystem outputs allowed");
+			  	shriek(454, "No filesystem outputs allowed");
 			  filename = limit_pathname(par, cfg->pseudo_root_dir);
 			  socket = open(filename, in == T_INPUT ? O_RDONLY | O_NONBLOCK | O_BINARY
 						: O_WRONLY | O_CREAT | O_TRUNC | O_NONBLOCK | O_BINARY, MODE_MASK);
 			  free(filename);
-			  if (socket == -1) shriek(445, fmt("Cannot open file %s", par));
+			  if (socket == -1) shriek(445, "Cannot open file %s", par);
 			  else close_upon_exit = true;
 			  break;
 		case '#': socket = special_io(par + 1, in);
@@ -503,14 +640,14 @@ a_io::a_io(const char *par, DATA_TYPE in, DATA_TYPE out) : agent(in, out)
 			/* if ever adding classes, take care of closing/nonclosing
 			 * the socket upon exit        */
 	}
-	if (socket >= 0) stretch_sleep_table(socket);
-	DEBUG(0,11,fprintf(STDDBG, "I/O agent is %p\n", this);)
+	if (socket >= 0) stretch_sleep_tables(socket);
+	D_PRINT(0, "I/O agent is %p\n", this);
 }
 
 a_io::~a_io()
 {
-	DEBUG(0,11,fprintf(STDDBG, "~a_io\n");)
-	if (close_upon_exit) async_close(socket);
+	D_PRINT(0, "~a_io\n");
+	if (close_upon_exit) close_and_invalidate(socket);
 }
 
 class a_input : public a_io
@@ -534,13 +671,18 @@ a_input::a_input(const char *par) : a_io(par, T_INPUT, T_TEXT)
 void a_input::run()
 {
 	int res;
-	DEBUG(0,11,fprintf(STDDBG, "Entering input agent\n");)
-	if (sleep_table[socket]) {
-		DEBUG(0,11,fprintf(STDDBG, "avoiding a nested input\n");)
+	D_PRINT(0, "Entering input agent\n");
+	if (block_table[socket]) {
+		D_PRINT(2, "avoiding a nested input\n");
 		block(socket);
 		return;
 	}
 	res = yread(socket, (char *)inb + offset, toread - offset);
+	if (res == -1 && errno == EAGAIN) {
+		D_PRINT(2, "avoiding an EAGAIN on input\n");
+		block(socket);
+		return;
+	}
 	if (res <= 0) {
 		if (!dc) {
 			if (res == 0) shriek(438, "end of file");
@@ -550,15 +692,17 @@ void a_input::run()
 		c->leave();
 		delete data_conns->remove(dc->handle);
 		c->enter();
-		shriek(436, fmt("data conn %d lost", socket));
+		shriek(436, "data conn %d lost", socket);
 	}
 	offset += res;
 	if (offset == toread) {
+		void *dta = inb;
+		((char *)dta)[offset] = 0;
 		switch (out) {
-			case T_DIPHS:
+			case T_SEGS:
 				if ((((segment *)inb)->code + 1) * (int)sizeof(segment) != offset)
-					shriek(432, fmt("Received bad segments: %d segs, %d bytes",
-						((segment *)inb)->code, offset));
+					shriek(432, "Received bad segments: %d segs, %d bytes",
+						((segment *)inb)->code, offset);
 				break;
 			case T_WAVEFM:
 				wavefm *w;
@@ -568,11 +712,11 @@ void a_input::run()
 				inb = NULL; toread = offset = 0;
 				pass(w);
 				return;
+			case T_TEXT:
+				encode_string((char *)inb, this_lang->charset, false);
 			default: ;	/* otherwise no problem */
 		}
-		void *dta = inb;
-		((char *)dta)[offset] = 0;
-		DEBUG(2,11,fprintf(STDDBG, "Read and about to process %s\n", (char *)dta);)
+		D_PRINT(2, "Read and about to process %s\n", (char *)dta);
 		inb = NULL;
 		toread = 0;	// superfluous
 		offset = 0;	// superfluous
@@ -583,13 +727,14 @@ void a_input::run()
 bool
 a_input::mktask(int size)
 {
-	DEBUG(2,11,fprintf(STDDBG, "%d bytes to be read\n", size);)
+	D_PRINT(2, "%d bytes to be read\n", size);
 	if (inb) return false;	// busy
 	toread = size;
+	D_PRINT(0, "Alloc in a_input:\n");
 	inb = xmalloc(size + 1);
 	offset = 0;
 	block(socket);
-	DEBUG(1,11,fprintf(STDDBG, "Apply task has been scheduled\n");)
+	D_PRINT(1, "Apply task has been scheduled\n");
 	return true;
 }
 
@@ -597,24 +742,31 @@ a_input::mktask(int size)
 class a_output : public a_io
 {
 	virtual int insize() = 0;
+	virtual void decode() {};
 	virtual void run();
 	virtual const char *name() { return "output"; };
 	bool foreground() {return ((stream *)next)->foreground(); };
 	int written;
+	bool decoded;
    protected:
 	void report(bool total, int written);
    public:
-	a_output(const char *par, DATA_TYPE i) : a_io(par, i, T_NONE) {written = 0;};
+	a_output(const char *par, DATA_TYPE i) : a_io(par, i, T_NONE) {written = 0; decoded = false;};
 };
 
 void
 a_output::run()
 {
-	if (sleep_table[socket]) {
+	if (push_table[socket]) {
 		push(socket);
 		return;
 	}
 	int size = insize();
+	if (!decoded) {
+		decode();
+		decoded = true;
+	}
+	
 	if (written) {
 		int now_written = ywrite(socket, (char *)inb + written, size - written);
 		report(false, now_written);
@@ -628,6 +780,7 @@ a_output::run()
 	}
 	if (written == size) {
 		written = 0;
+		decoded = false;
 		relax();
 		finis(false);
 	} else push(socket);
@@ -640,38 +793,41 @@ a_output::report(bool total, int written)
 	if (foreground()) {
 		reply(total ? "122 total bytes" : "123 written bytes");
 		sprintf(scratch, " %d", written);
-		sputs(scratch, cfg->sd_out);
-		sputs("\r\n", cfg->sd_out);
+		sputs(scratch, cfg->get__sd_out());
+		sputs("\r\n", cfg->get__sd_out());
 	}
 }
 
 
-class oa_ascii : public a_output
+template <DATA_TYPE type> class oa_ascii : public a_output
 {
 	virtual int insize() {
 		return strlen((char *)inb);
 	}
+	virtual void decode() {
+		decode_string((char *)inb, this_lang->charset);
+	}
    public:
-	oa_ascii(const char *s): a_output(s, T_TEXT) {};
+	oa_ascii(const char *s): a_output(s, type) {};
 };
 
-class oa_stml : public a_output
-{
-	virtual int insize() {
-		return strlen((char *)inb);
-	}
-   public:
-	oa_stml(const char *s): a_output(s, T_STML) {};
-};
+//class oa_stml : public a_output
+//{
+//	virtual int insize() {
+//		return strlen((char *)inb);
+//	}
+//   public:
+//	oa_stml(const char *s): a_output(s, T_STML) {};
+//};
 
 class oa_seg : public a_output
 {
 	virtual int insize() {
-		DEBUG(1,11,fprintf(STDDBG, "Sending %d segments\n", ((segment *)inb)->code);)
+		D_PRINT(1, "Sending %d segments\n", ((segment *)inb)->code);
 		return ((segment *)inb)->code * sizeof(segment);
 	}
    public:
-	oa_seg(const char *s): a_output(s, T_DIPHS) {};
+	oa_seg(const char *s): a_output(s, T_SEGS) {};
 };
 
 class oa_wavefm : public a_output
@@ -680,7 +836,7 @@ class oa_wavefm : public a_output
 		shriek(462, "abstract oa_wavefm::insize"); return 0;
 	}
 	virtual void run();
-	virtual void brk();
+	virtual bool brk();
 	bool attached;
    public:
 	oa_wavefm(const char *s): a_output(s, T_WAVEFM) {attached = false;};
@@ -691,9 +847,9 @@ oa_wavefm::run()
 {
 	wavefm *w = (wavefm *)inb;
 	
-	if (!attached && !sleep_table[socket]) {
+	if (!attached && !push_table[socket]) {
 		w->attach(socket);
-		report(false, w->written /* + sizeof(wave_header)*/);
+		report(false, w->written);
 		attached = true;
 	}
 	bool to_do;
@@ -706,7 +862,7 @@ oa_wavefm::run()
 		if (w->written) report(false, w->written);
 		report(true, w->written_bytes());
 		w->detach(socket);
-		DEBUG(1,11,fprintf(STDDBG, "oa_wavefm wrote %d bytes\n", w->written_bytes());)
+		D_PRINT(1, "oa_wavefm wrote %d bytes\n", w->written_bytes());
 		attached = false;
 		delete w;
 		inb = NULL;
@@ -714,21 +870,22 @@ oa_wavefm::run()
 	}
 }
 
-void
+bool
 oa_wavefm::brk()
 {
 	if (inb) {
 
 		wavefm *w = (wavefm *)inb;
 		
-		report(false, w->written_bytes());
+		report(true, w->written_bytes());
 		w->brk();
 		if (attached) w->detach(socket);
-		DEBUG(1,11,fprintf(STDDBG, "oa_wavefm wrote %d bytes\n", w->written_bytes());)
+		D_PRINT(1, "oa_wavefm wrote %d bytes\n", w->written_bytes());
 		attached = false;
 		relax();
 	}
 	finis(true);
+	return true;
 }
 
 
@@ -737,19 +894,21 @@ oa_wavefm::brk()
  *	stream agent itself. stream->head is an input agent.
  */
 
-enum agent_type {AT_UNKNOWN, AT_CHUNK, AT_JOIN, AT_ASCII, AT_DIPHS, AT_PRINT, AT_RULES, AT_STML, AT_SYNTH,
-		 AT_T_TEXT, AT_T_STML, AT_T_UNITS, AT_T_DIPHS, AT_T_WAVEFM};
-const char *agent_type_str = ":chunk:join:raw:diphs:print:rules:stml:synth:[t]:[s]:[i]:[d]:[w]:";
+enum agent_type {AT_UNKNOWN, AT_CHUNK, AT_JOIN, AT_ASCII, AT_SSIF, AT_SEGS,
+			AT_PRINT, AT_RULES, AT_STML, AT_SYN, AT_TRAD_SYNTH,
+			AT_T_TEXT, AT_T_STML, AT_T_UNITS, AT_T_SSIF, AT_T_SEGS, AT_T_WAVEFM};
+const char *agent_type_str = ":chunk:join:raw:dump:diphs:print:rules:stml:syn:synth:[t]:[s]:[i]:[p]:[d]:[w]:";
 
 agent *make_agent(char *s, agent *preceding)
 {
 	if (strchr("@#/.$", *s)) {
 		if (!preceding) return new a_input(s);
 		switch (preceding->out) {
-			case T_TEXT:   return new oa_ascii(s);
-			case T_STML:   return new oa_stml(s);
+			case T_TEXT:   return new oa_ascii<T_TEXT>(s);
+			case T_STML:   return new oa_ascii<T_STML>(s);
 			case T_UNITS: shriek(448, "Units are hard to output");
-			case T_DIPHS:  return new oa_seg(s);
+			case T_SSIF:   return new oa_ascii<T_SSIF>(s);
+			case T_SEGS:  return new oa_seg(s);
 			case T_WAVEFM: return new oa_wavefm(s);
 			default: shriek(462, "unimplmd oa");
 		}
@@ -759,20 +918,23 @@ agent *make_agent(char *s, agent *preceding)
 		case AT_UNKNOWN: shriek(861, "Agent type bug.");
 		case AT_ASCII: return new a_ascii;
 		case AT_CHUNK: return new a_chunk;
-		case AT_DIPHS: return new a_segs;
+		case AT_SSIF:  return new a_ssif;
+		case AT_SEGS:  return new a_segs;
 		case AT_JOIN:  return new a_join;
 		case AT_PRINT: return new a_print;
 		case AT_RULES: return new a_rules;
 		case AT_STML:  return new a_stml;
-		case AT_SYNTH: return new a_synth;
+		case AT_SYN:   return new a_syn;
+		case AT_TRAD_SYNTH: return new a_synth;
 
 		case AT_T_TEXT:  return new a_type<T_TEXT>;
 		case AT_T_STML:  return new a_type<T_STML>;
 		case AT_T_UNITS: return new a_type<T_UNITS>;
-		case AT_T_DIPHS: return new a_type<T_DIPHS>;
+		case AT_T_SSIF:  return new a_type<T_SSIF>;
+		case AT_T_SEGS:  return new a_type<T_SEGS>;
 		case AT_T_WAVEFM:return new a_type<T_WAVEFM>;
 
-		default:       shriek(415, "Unknown agent type"); return NULL;
+		default:       shriek(415, "Unknown agent type %s", s); return NULL;
 	}
 }
 
@@ -784,16 +946,14 @@ stream::stream(char *s, context *pc) : agent(T_NONE, T_NONE)
 	agent *l = head = NULL;
 
 	callbk = NULL;
-//	c = new context(cfg->sd_in, cfg->sd_out);	// a little redundant
 	c = pc;
-//	navelcord<context> ncc(c);
 
 	tmp = strchr(s, LIST_DELIM);
 	if (!tmp) shriek(415, "Bad stream syntax");
 
 	do {
 		*tmp = 0;
-		DEBUG(1,11,fprintf(STDDBG, "Making agent out of %s\n", s);)
+		D_PRINT(1, "Making agent out of %s\n", s);
 		try {
 			a = make_agent(s, NULL); a->c = c;
 		} catch (command_failed *e) {
@@ -812,7 +972,6 @@ stream::stream(char *s, context *pc) : agent(T_NONE, T_NONE)
 	a->prev = l;
 	a->next = this;
 	if (head->next != this) head->out = head->next->in;	/* adjust a_input type */
-//	ncc.release();
 }
 
 stream::~stream()
@@ -834,18 +993,19 @@ stream::release_agents()
 void
 stream::apply(agent *ref, int bytes)
 {
-	DEBUG(2,11,fprintf(STDDBG, "In stream::apply %p %p %d\n", head, ref, bytes);)
+	D_PRINT(2, "In stream::apply %p %p %d\n", head, ref, bytes);
 	callbk = ref;
 	head->mktask(bytes);
 }
 
-void
+bool
 stream::brk()
 {
-	if (!callbk) return;	/* break only if running */
-	reply("401 interrupted");
+	if (!callbk) return false;	/* break only if running */
 	for (agent *a = head; a && a != this; a = a->next)
 		a->brk();
+	reply("401 interrupted");
+	return true;
 }
 
 void
@@ -857,13 +1017,14 @@ stream::run()
 void
 stream::finis(bool err)		// FIXME: simplify
 {
-	DEBUG(2,11,fprintf(STDDBG, "submitted a subtask\n");)
+	if (err) reply("191 finis recovery");
+	D_PRINT(2, "submitted a subtask\n");
 	for (agent *a = head; a != this; a = a->next) {
 		if (a->inb || a->pendin) {
-			DEBUG(2,11,fprintf(STDDBG, "more subtasks are pending\n");)
+			D_PRINT(2, "more subtasks are pending\n");
 			if (err) {
 				a->relax();
-				DEBUG(2,11,fprintf(STDDBG, "subtasks discarded\n");)
+				D_PRINT(2, "subtasks discarded\n");
 				if  (callbk) callbk->schedule();
 				else shriek(862, "double fault - no callback");
 				callbk = NULL;
@@ -872,7 +1033,7 @@ stream::finis(bool err)		// FIXME: simplify
 			return;
 		}
 	}
-	DEBUG(2,11,fprintf(STDDBG, "this has been the last subtask\n");)
+	D_PRINT(2, "this has been the last subtask\n");
 	if (!err) reply("200 output OK");
 	if (callbk) callbk->schedule();
 	else shriek(862, "no callback");
@@ -910,6 +1071,7 @@ void a_disconnector::run()
 	if (!last) shriek(861, "Spurious disconnect");
 	delete to_delete[--last];
 	to_delete[last] = NULL;
+	D_PRINT(0, "Disconnect finished, last is %d\n", last);
 }
 
 void a_disconnector::disconnect(a_protocol *moriturus)
@@ -919,6 +1081,7 @@ void a_disconnector::disconnect(a_protocol *moriturus)
 		to_delete = (a_protocol **)xrealloc(to_delete, max * sizeof(void *));
 	}
 	to_delete[last++] = moriturus;
+	D_PRINT(0, "Disconnect requested, last is %d\n", last);
 	schedule();
 }
 
@@ -940,40 +1103,44 @@ a_protocol::~a_protocol()
 void a_protocol::run()
 {
 	int res;
-	res = sgets(buffer, cfg->max_net_cmd, cfg->sd_in, sgets_buff);
+	res = sgets(buffer, cfg->max_net_cmd, cfg->get__sd_in(), sgets_buff);
 	if (res < 0) {
 		disconnect();
 		return;
 	}
 
+//	encode_string(buffer, this_lang->charset, false);	// FIXME (alloc->true)
+
+	encode_string(buffer, this_lang->charset, false);	// FIXME (alloc->true)
+
 	if ((int)strlen(buffer) >= cfg->max_net_cmd)
 		shriek(413, "Received command is too looong");
 	if (res > 0 && *buffer) switch (run_command(buffer)) {
 		case PA_NEXT:
-			DEBUG(0,11,fprintf(STDDBG, "PA_NEXT\n");)
+			D_PRINT(0, "PA_NEXT\n");
 			if (strchr(sgets_buff, '\n')) schedule();
-			else block(cfg->sd_in);
+			else block(cfg->get__sd_in());
 			return;
 		case PA_DONE:
-			DEBUG(0,11,fprintf(STDDBG, "PA_DONE\n");)
+			D_PRINT(0, "PA_DONE\n");
 			disconnect();
 			return;
 		case PA_WAIT:
-			DEBUG(0,11,fprintf(STDDBG, "PA_WAIT\n");)
+			D_PRINT(0, "PA_WAIT\n");
 			return;
 		default:
 			shriek(861, "Bad protocol action\n");
 	}
-	block(cfg->sd_in);		/* partial line read */
+	block(cfg->get__sd_in());		/* partial line read */
 
 //	leave_context(i);
 
 //	non-blocking get_line etc.
 }
 
-a_ttscp::a_ttscp(int sd_in, int sd_out) : a_protocol()
+a_ttscp::a_ttscp(socky int _sd_in, socky int _sd_out) : a_protocol()
 {
-	c = new context(sd_in, sd_out);
+	c = new context(_sd_in, _sd_out);
 	c->enter();
 	
 	handle = (char *)malloc(cfg->handle_size + 1);
@@ -987,14 +1154,14 @@ a_ttscp::a_ttscp(int sd_in, int sd_out) : a_protocol()
 		"extensions:\r\n"
 		"server: Epos\r\n"
 		"release: " VERSION "\r\n"
-		"handle: ", cfg->sd_out);
-	sputs(		handle, cfg->sd_out);
-	sputs(	"\r\n", cfg->sd_out);
+		"handle: ", cfg->get__sd_out());
+	sputs(		handle, cfg->get__sd_out());
+	sputs(	"\r\n", cfg->get__sd_out());
 	ctrl = NULL;
 	deps = new hash_table<char, a_ttscp>(4);
 	deps->dupdata = deps->dupkey = false;
 	c->leave();
-	block(sd_in);
+	block(_sd_in);
 }
 
 /*
@@ -1007,7 +1174,7 @@ a_ttscp::~a_ttscp()
 	c->enter();
 	if (cfg->current_stream) delete cfg->current_stream;
 	cfg->current_stream = NULL;
-	DEBUG(2,11,fprintf(STDDBG, "deleted context closes fd %d and %d\n", cfg->sd_in, cfg->sd_out);)
+	D_PRINT(2, "deleted context closes fd %d and %d\n", cfg->get__sd_in(), cfg->get__sd_out());
 	c->leave();
 	while (deps->items) {
 		a_ttscp *tmp = deps->translate(deps->get_random());
@@ -1016,10 +1183,10 @@ a_ttscp::~a_ttscp()
 	}
 	delete deps;
 	c->enter();
-	if (cfg->sd_in != -1)
-		async_close(cfg->sd_in);
-	if (cfg->sd_out != -1 && cfg->sd_out != cfg->sd_in)
-		async_close(cfg->sd_out);
+	if (cfg->get__sd_in() != -1)
+		close_and_invalidate(cfg->get__sd_in());
+	if (cfg->get__sd_out() != -1 && cfg->get__sd_out() != cfg->get__sd_in())
+		close_and_invalidate(cfg->get__sd_out());
 	if (data_conns->translate(handle) || ctrl_conns->translate(handle))
 		shriek(862, "Forgot to forget a_ttscp");
 
@@ -1029,11 +1196,12 @@ a_ttscp::~a_ttscp()
 	// close the descriptor? no, the ~context does that
 }
 
-void
+bool
 a_ttscp::brk()
 {
 	if (c->config->current_stream)
-		c->config->current_stream->brk();
+		return c->config->current_stream->brk();
+	return false;
 }
 
 int
@@ -1042,7 +1210,7 @@ a_ttscp::run_command(char *cmd)
 	char *keyword;
 	char *param;
 
-	DEBUG(2,11,fprintf(STDDBG, "[ cmd] %s\n", cmd);)
+	D_PRINT(2, "[ cmd] %s\n", cmd);
 
 	keyword = cmd + strspn (cmd, WHITESPACE);
 	param = keyword + strcspn(keyword, WHITESPACE);
@@ -1071,14 +1239,14 @@ a_ttscp::run_command(char *cmd)
 	try {
 		return ttscp_cmd_set[i].impl(param, this);
 	} catch (command_failed *e) {
-		DEBUG(2,11,fprintf(STDDBG, "Command failed, %d, %.60s\n", e->code, e->msg);)
+		D_PRINT(2, "Command failed, %d, %.60s\n", e->code, e->msg);
 		reply(e->code, e->msg);
 		delete e;
 		return PA_NEXT;
 	} catch (connection_lost *d) {
-		DEBUG(2,11,fprintf(STDDBG, "Releasing a TTSCP control connection, %d, %.60s\n", d->code, d->msg);)
+		D_PRINT(2, "Releasing a TTSCP control connection, %d, %.60s\n", d->code, d->msg);
 		reply(d->code, d->msg);		/* just in case */
-		reply(201, fmt("debug %d", cfg->sd_in));
+		reply(201, fmt("debug %d", cfg->get__sd_in()));
 		delete d;
 		return PA_DONE;
 	}
@@ -1091,7 +1259,7 @@ a_ttscp::run_command(char *cmd)
 void
 a_ttscp::disconnect()
 {
-	DEBUG(2,11,fprintf(STDDBG, "ctrl conn %d lost\n", cfg->sd_in);)
+	D_PRINT(2, "ctrl conn %d lost\n", cfg->get__sd_in());
 	if (this != ctrl_conns->remove(handle) /* && this != data_conns->remove(handle) */ )
 		shriek(862, "Failed to disconnect a ctrl connection");
 	disconnector.disconnect(this);
@@ -1100,14 +1268,14 @@ a_ttscp::disconnect()
 void make_nonblocking(int f)
 {
 #ifdef HAVE_WINSOCK
-	ioctlsocket((unsigned long)f, FIONBIO, (unsigned long *)&make_nonblocking);	// &make_nonblocking is a dummy non-NULL pointer
+	ioctlsocket((unsigned long int)f, FIONBIO, (unsigned long int *)&make_nonblocking);	// &make_nonblocking is a dummy non-NULL pointer
 #else
 	fcntl(f, F_SETFL, O_NONBLOCK);
 #endif
 }
 
 #ifndef HAVE_GETHOSTNAME
-int gethostbyname(char *b, size_t)
+int gethostname(char *b, size_t)
 {
 	strcpy(b, "localhost");
 	return 0;
@@ -1120,54 +1288,67 @@ a_accept::a_accept() : agent(T_NONE, T_NONE)
 	
 	char one = 1;
 
-	c = new context(-1, /*** dark errors ***/ DARK_ERRLOG);	//FIXME
-	c->enter();
+//	c = new context(-1, /*** dark errors ***/ DARK_ERRLOG);	//FIXME
+//	c->enter();
 
-	cfg->sd_in = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
+	listening = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
 	memset(&sa, 0, sizeof(sa));
-	gethostname(scratch, cfg->scratch - 1);
+	gethostname(scratch, scfg->scratch_size - 1);
 	sa.sin_family = AF_INET;
-	sa.sin_addr.s_addr = htonl(cfg->local_only ? INADDR_LOOPBACK : INADDR_ANY);
-	sa.sin_port = htons(cfg->listen_port);
-	setsockopt(cfg->sd_in, SOL_SOCKET, SO_REUSEADDR, &one, sizeof(int));
-	DEBUG(2,11,fprintf(STDDBG, "[core] Trying to bind...\n");)
-	if (bind(cfg->sd_in, (sockaddr *)&sa, sizeof (sa))) shriek(871, "Could not bind");
-	if (listen(cfg->sd_in, 64)) shriek(871, "Could not listen");
-	make_nonblocking(cfg->sd_in);
+	sa.sin_addr.s_addr = htonl(scfg->local_only ? INADDR_LOOPBACK : INADDR_ANY);
+	sa.sin_port = htons(scfg->listen_port);
+	setsockopt(listening, SOL_SOCKET, SO_REUSEADDR, &one, sizeof(int));
+	D_PRINT(3, "* Binding to the TTSCP port %d.\n", scfg->listen_port);
+	if (bind(listening, (sockaddr *)&sa, sizeof (sa))) shriek(871, "Could not bind");
+	if (listen(listening, 64)) shriek(871, "Could not listen");
+	make_nonblocking(listening);
 
 	ia.sin_family = AF_INET;
 	ia.sin_addr.s_addr = htonl(INADDR_ANY);
 	ia.sin_port = 0;
 
-	block(cfg->sd_in);
-	c->leave();
+	block(listening);
+//	c->leave();
 }
 
 a_accept::~a_accept()
 {
-	close (cfg->sd_in);
-	delete c;
+	close (listening);
+//	close (c->config->_sd_in);
+//	delete c;
 }
 
 void
 a_accept::run()
 {
 	static socklen_t sia = sizeof(sockaddr);	// Will __QNX__ complain?
-	int f = accept(cfg->sd_in, (sockaddr *)&ia, &sia);
+	int f = accept(listening, (sockaddr *)&ia, &sia);
 	if (f == -1) {
-//		shriek(871, fmt("Cannot accept() - network problem (errno %d)", errno));
-		DEBUG(3,11,fprintf(STDDBG, "Cannot accept() - errno %d! Madly looping.\n", errno);)
+//		shriek(871, "Cannot accept() - network problem (errno %d)", errno);
+		D_PRINT(3, "Cannot accept() - errno %d! Madly looping.\n", errno);
 		if (errno != EAGAIN) schedule();
 		return;
 	}
 	make_nonblocking(f);
-	DEBUG(2,11,fprintf(STDDBG, "[core] Accepted %d (on %d).\n", f, cfg->sd_in);)
-	c->leave();
+	D_PRINT(2, "Accepted %d (on %d).\n", f, listening);
+//	c->leave();
 	unuse(new a_ttscp(f, f));
-	c->enter();
-	block(cfg->sd_in);
+//	c->enter();
+	block(listening);
 }
 
+struct sched_aq
+{
+	agent *ag;
+	sched_aq *next;
+	sched_aq *prev;
+
+	void *operator new(size_t size);
+	void operator delete(void *ptr);
+};
+
+SLABIFY(sched_aq, sched_aq_slab, 341, shutdown_sched_aq);
+
 sched_aq *sched_head = NULL;
 sched_aq *sched_tail = NULL;
 
@@ -1178,6 +1359,7 @@ agent::schedule()
 {
 	if (!this) shriek(862, "scheduling garbage");
 	runnable_agents++;
+	D_PRINT(1, "%d runnable agents\n", runnable_agents);
 	sched_aq *tmp = new sched_aq;
 	tmp->ag = this;
 	tmp->prev = NULL;
@@ -1199,21 +1381,31 @@ agent *sched_sel()
 	sched_tail = sched_tail->prev;
 	delete tmp;
 	runnable_agents--;
-	DEBUG(1,11,fprintf(STDDBG, "Agent %s\n", r->name());)
+	D_PRINT(1, "Agent %s\n", r->name());
 	return r;
 }
 
-agent **sleep_table = (agent **)xmalloc(1);
+void shutdown_agent_queue()
+{
+	for (sched_aq *tmp = sched_head; tmp; tmp = tmp->next)
+		delete tmp;
+}
+
+agent **block_table = (agent **)xmalloc(1);
+agent **push_table = (agent **)xmalloc(1);
 fd_set block_set;
 fd_set push_set;
 socky int select_fd_max = 0;
 
-void stretch_sleep_table(socky int fd)
+void stretch_sleep_tables(socky int fd)
 {
 	if (select_fd_max <= fd) {
-		sleep_table = (agent **)xrealloc(sleep_table, (fd + 1) * sizeof(agent *));
-		for ( ; select_fd_max <= fd; select_fd_max++)
-			sleep_table[select_fd_max] = NULL;
+		block_table = (agent **)xrealloc(block_table, (fd + 1) * sizeof(agent *));
+		push_table = (agent **)xrealloc(push_table, (fd + 1) * sizeof(agent *));
+		for ( ; select_fd_max <= fd; select_fd_max++) {
+			block_table[select_fd_max] = NULL;
+			push_table[select_fd_max] = NULL;
+		}
 	}
 }
 
@@ -1224,19 +1416,19 @@ void stretch_sleep_table(socky int fd)
 void
 agent::block(socky int fd)
 {
-	DEBUG(1,11,fprintf(STDDBG, "Sleeping on %d\n", fd);)
-	stretch_sleep_table(fd);
-	if (sleep_table[fd]) {
+	D_PRINT(1, "Sleeping on %d\n", fd);
+	stretch_sleep_tables(fd);
+	if (block_table[fd]) {
 		agent *a;
 
-		if (this == sleep_table[fd])
-			shriek(861, fmt("Resleeping on %d", fd));
+		if (this == block_table[fd])
+			shriek(861, "Resleeping on %d", fd);
 		if (!FD_ISSET(fd, &block_set))
-			shriek(861, fmt("Countersleeping on %d", fd));
-		for (a = sleep_table[fd]; a->dep; a = a->dep) ;
+			shriek(861, "Countersleeping on %d", fd);
+		for (a = block_table[fd]; a->dep; a = a->dep) ;
 		a->dep = this;
 	} else {
-		sleep_table[fd] = this;
+		block_table[fd] = this;
 		FD_SET(fd, &block_set);
 	}
 	return;
@@ -1245,25 +1437,21 @@ agent::block(socky int fd)
 void
 agent::push(socky int fd)
 {
-	DEBUG(1,11,fprintf(STDDBG, "Pushing on %d\n", fd);)
-	stretch_sleep_table(fd);
-	if (sleep_table[fd]) {
+	D_PRINT(1, "Pushing on %d\n", fd);
+	stretch_sleep_tables(fd);
+	if (push_table[fd]) {
 		agent *a;
 
-		if (this == sleep_table[fd])
-			shriek(861, fmt("Resleeping on %d", fd));
+		if (this == push_table[fd])
+			shriek(861, "Resleeping on %d", fd);
 		if (!FD_ISSET(fd, &push_set))
-			shriek(861, fmt("Countersleeping on %d", fd));
-		for (a = sleep_table[fd]; a->dep; a = a->dep) ;
+			shriek(861, "Countersleeping on %d", fd);
+		for (a = push_table[fd]; a->dep; a = a->dep) ;
 		a->dep = this;
 	} else {
-		sleep_table[fd] = this;
+		push_table[fd] = this;
 		FD_SET(fd, &push_set);
 	}
 	return;
 }
 
-//void free_sleep_table()
-//{
-//	free(sleep_table);
-//}
