@@ -14,8 +14,7 @@
  *
  */
 
-
-#include "common.h"
+#include "epos.h"
 #include "exc.cc"
 
 #ifdef HAVE_TIME_H
@@ -38,6 +37,17 @@
 	#endif
 #endif
 
+#ifndef HAVE_VSNPRINTF
+int vsnprintf(char *str, size_t size, const char *format, ...)
+{
+	va_list ap;
+	va_start(ap, format);
+	int ret = vsprintf(str, format, ap);
+	va_end(ap);
+	return ret;
+}
+#endif	// ifdef HAVE_VSNPRINTF
+
 #ifdef HAVE_IO_H
 	#include <io.h>
 #endif
@@ -62,7 +72,8 @@
 
 char *scratch = NULL;
 
-char *esctab = NULL;
+charxlat *esctab = NULL;
+const char* WHITESPACE    = " \t\r";     //i.e. space and tab
 
 int unused_variable;
 
@@ -75,7 +86,7 @@ int severity(int code)
 	if (code / 10 == 80) return LOG_NOTICE;
 	if (code >= 610) return LOG_CRIT;
 	if (code / 10 == 45 || code == 444 || code == 445) {
-		if (cfg->authpriv) return LOG_ERR | LOG_AUTHPRIV;
+		if (scfg->authpriv) return LOG_ERR | LOG_AUTHPRIV;
 		return LOG_WARNING;
 	}
 	if (code / 10 == 46) return LOG_ERR;
@@ -86,113 +97,92 @@ int severity(int code)
 }
 #endif
 
+char error_fmt_scratch[MAX_ERR_LINE];
+int errors = 0;
+int fatal_bugs = 0;
 
-void shriek(int code, const char *s) 
-{	/* Art (c) David Miller */
-	if (cfg->shriek_art == 1) fprintf(cfg->stdshriek,
+void shriek(int code, const char *fs, ...) 
+{
+	int l_errno = errno;
+	/* Art (c) David Miller */
+	if (scfg->shriek_art == 1) fprintf(cfg->stdshriek,
 "              \\|/ ____ \\|/\n"
 "              \"@'/ ,. \\`@\"\n"
 "              /_| \\__/ |_\\\n"
 "                 \\__U_/\n");
-	if (cfg->shriek_art == 2) fprintf(cfg->stdshriek, "\nSuddenly, something went wrong.\n\n");
-	color(cfg->stdshriek, cfg->shriek_col);
-	fprintf(cfg->stdshriek, "Fatal: %s\n",s); 
-	color(cfg->stdshriek, cfg->normal_col);
+	if (scfg->shriek_art == 2) fprintf(cfg->stdshriek, "\nSuddenly, something went wrong.\n\n");
 
+	const char *message;
+	if (strchr(fs, '%')) {
+		va_list ap;
+		va_start(ap, fs);
+		vsnprintf(error_fmt_scratch, MAX_ERR_LINE, fs, ap);
+		error_fmt_scratch[MAX_ERR_LINE - 1] = 0;
+		message = error_fmt_scratch;
+		va_end(ap);
+	} else message = fs;
+
+	color(cfg->stdshriek, scfg->fatal_color);
+	fprintf(cfg->stdshriek, "Error: %s (%d)\n", message, code); 
+	color(cfg->stdshriek, scfg->normal_color);
 
 #ifdef HAVE_SYSLOG_H
-	if (cfg->use_syslog)
-		if (cfg->log_codes) syslog(LOG_DAEMON | severity(code), "%3d %s", code, s);
-		else syslog(LOG_DAEMON | severity(code), "%s", s);
+	if (scfg->syslog)
+		if (scfg->log_codes) syslog(LOG_DAEMON | severity(code), "%3d %s", code, message);
+		else syslog(LOG_DAEMON | severity(code), "%s", message);
+	unuse(l_errno);
 #else
-	FILE *hackfile = fopen("hackfile","w");
-	if (hackfile) {
-		fprintf(hackfile, s);
-		fclose(hackfile);
+	FILE *h = fopen("epos.err","w");
+	if (h) {
+		fprintf(h, "%s\nerrno=%d (%s)\n", message, l_errno, strerror(l_errno));
+		fclose(h);
 	}
 #endif
 	command_failed *xcf;
 
 	switch (code / 100) {
 		case 4 :
-//			printf("Just a command will fail.\n");
-			xcf = new command_failed (code, s);
+			xcf = new command_failed (code, message);
 			throw xcf;
 
 		case 6 :
-			throw new connection_lost (code, s);
+			throw new connection_lost (code, message);
 
 		case 8 :
-			printf("Abnormal condition: %s (code %d)\n", s, code);
-			if (errno && EAGAIN) printf("Current errno value: %d (%s)\n", errno, strerror(errno));
-			throw new fatal_error (code, s);
+			errors++;
+			if (code / 10 == 86) fatal_bugs++;
+			if (l_errno && EAGAIN) printf("Current errno value: %d (%s)\n", l_errno, strerror(l_errno));
+			throw new fatal_error (code, message);
 
 			printf("Your compiler does NOT support exception handling, aborting\n");
 			fflush(NULL);
 			abort();
-		default:  shriek(869, fmt("Bad error class %d", code));
+		default:  shriek(869, "Bad error class %d", code);
 	}
 }
 
-char error_fmt_scratch[MAX_ERR_LINE];
-
-char *fmt(const char *s, int i) 
+char *fmt(const char *s, ...) 
 {
-	sprintf(error_fmt_scratch, s, i);
-	return error_fmt_scratch;
+	va_list ap;
+	va_start(ap, s);
+	vsnprintf(scratch, scfg->scratch_size, s, ap);
+	va_end(ap);
+	return scratch;
 }
 
-char *fmt(const char *s, const char *t, const char *u) 
+void tag_vfprintf(const char *fmt, va_list ap)
 {
-	sprintf(error_fmt_scratch, s, t, u);
-	return error_fmt_scratch;
+	if (!fmt || !*fmt) return;
+//	if (cfg->debug_tag) fprintf(STDDBG, "%-9s ", module_debug_tag);
+	vfprintf(STDDBG, fmt, ap);
 }
 
-char *fmt(const char *s, const char *t, int i)
+void dprint_always(int level, const char *fmt, ...)
 {
-	sprintf(error_fmt_scratch, s, t, i);
-	return error_fmt_scratch;
-}
-
-char *fmt(const char *s, const char *t)
-{
-	sprintf(error_fmt_scratch,s,t);
-	return error_fmt_scratch;
-}
-
-char *fmt(const char *s, const char *t, const char *u, int i)
-{
-	sprintf(error_fmt_scratch, s, t, u, i);
-	return error_fmt_scratch;
-}
-
-char *fmt(const char *s, int i, int j)
-{
-	sprintf(error_fmt_scratch, s, i, j);
-	return error_fmt_scratch;
-}
-
-char *fmt(const char *s, const char *t, const char *u, const char *v)
-{
-	sprintf(error_fmt_scratch, s, t, u, v);
-	return error_fmt_scratch;
-}
-
-char *fmt(const char *s, int t, const char *u)
-{
-	sprintf(error_fmt_scratch, s, t, u);
-	return error_fmt_scratch;
-}
-
-char *fmt(const char *s, int t, const char *u, const char *v)
-{
-	sprintf(error_fmt_scratch, s, t, u, v);
-	return error_fmt_scratch;
-}
-
-void hash_shriek(const char *s1, const char *s2, int i)
-{
-	shriek(812, fmt(s1, s2, i));	//FIXME?
+	va_list ap;
+	va_start(ap, fmt);
+	tag_vfprintf(fmt, ap);
+	va_end(ap);
 }
 
 void user_pause()
@@ -224,15 +214,15 @@ fopen(const char *filename, const char *flags, const char *reason)
 	switch (*flags) {
 		case 'r': message = "Failed to read %s from %s: %s"; break;
 		case 'w': message = "Failed to write %s to %s: %s"; break;
-		default : shriek(861, fmt("Bad flags for %s", reason)); message = NULL;
+		default : shriek(861, "Bad flags for %s", reason); message = NULL;
 	}
 	f = fopen(filename, flags);
 	if (!f && errno == ENOMEM)
 		OOM_HANDLER;
-	if (!f && reason) shriek(445, fmt(message, reason, filename, strerror(errno)));
+	if (!f && reason) shriek(445, message, reason, filename, strerror(errno));
 	if (cfg->paranoid && f && *flags == 'r') {
-		if (!fread(&message, 1, 1, f))
-			shriek(445, fmt(message, reason, filename, "maybe a directory"));
+		if (reason && !fread(&message, 1, 1, f))
+			shriek(445, message, reason, filename, "maybe a directory");
 		fseek(f, 0, SEEK_SET);
 	}
 	return f;
@@ -240,12 +230,12 @@ fopen(const char *filename, const char *flags, const char *reason)
 
 void colorize(int level, FILE *handle)
 {
-	if (!cfg->colored) return; 
+	if (!scfg->colored) return; 
 	if (level==-1) {
-		fputs(cfg->normal_col, handle);
+		if (scfg->normal_color) fputs(scfg->normal_color, handle);
 		return;
 	}
-	fputs(cfg->out_color[level],handle);
+	if (scfg->color[level]) fputs(scfg->color[level],handle);
 }
 
 FIT_IDX fit(char c)
@@ -257,7 +247,7 @@ FIT_IDX fit(char c)
 		case 'I': return Q_INTENS;
 		case 't':
 		case 'T': return Q_TIME;
-		default: shriek(811, fmt("Expected f,i or t, found %c",c));
+		default: shriek(811, "Expected f,i or t, found %c",c);
 			return Q_FREQ;
 	}
 }
@@ -277,10 +267,10 @@ UNIT str2enum(const char *item, const char *list, int dflt)
 			k++, j=item-1, i=strchr(i,LIST_DELIM);
 			if (!i) break;
 		}
-//		DEBUG(0,0,fprintf(STDDBG,"str2enum %s %s %d\n",i,j,k);)
+//		D_PRINT(0, "str2enum %s %s %d\n",i,j,k);
 	}
 	if(i && j && !*j) return (UNIT)k;
-	DEBUG(3,0,fprintf(STDDBG,"str2enum returns ILL for %s %s\n",item, list);)
+	D_PRINT(2, "str2enum returns ILL for %s %s\n",item, list);
 	return U_ILL;
 }
 
@@ -301,7 +291,7 @@ const char *enum2str(int item, const char *list)
 		} else {
 			_enum2str_buff[j++] = *i;
 			if (j >= MAX_SYMBOLIC)
-				shriek(461, fmt("Symbolic %.20s... too long", _enum2str_buff));
+				shriek(461, "Symbolic %.20s... too long", _enum2str_buff);
 		}
 	}
 
@@ -310,7 +300,27 @@ const char *enum2str(int item, const char *list)
 		_enum2str_buff[j] = 0;
 		return _enum2str_buff;
 	}
-	return NULL;	// shriek(446, fmt("enum2str should stringize %d",item));
+	return NULL;	// shriek(446, "enum2str should stringize %d",item);
+}
+
+void list_of_calls(const char *list, void call(int, const char *))
+{
+	int i;
+	int j = 0;
+	int k = 0;
+	char *tmp = (char *)xmalloc(strlen(list)+1);
+
+
+	for (i=0; (tmp[j] = list[i]); i++) {
+		if (tmp[j] == ':' ) {
+			tmp[j] = 0;
+			call(k, tmp);
+			j = 0;
+			k++;
+		} else j++;
+	}
+	call(k, tmp);
+	free(tmp);
 }
 
 /*************
@@ -327,7 +337,7 @@ hash *str2hash(const char *list, unsigned int max_item_len)
 		item=enum2str(d-1, list);	// sllooww
 		h->add_int(item,d-1);
 		if (max_item_len && strlen(item) > max_item_len)
-			shriek(446, fmt("Directive %s too long", item));
+			shriek(446, "Directive %s too long", item);
 	}
 	return h;
 }
@@ -345,16 +355,16 @@ unit *str2units(const char *text)
 	unit *root;
 	parser *parsie;
 
-	if (text && (signed)strlen(text) > cfg->maxtext) shriek(456, "input too long");
+	if (text && (signed)strlen(text) > scfg->max_text_size) shriek(456, "input too long");
 
-	if (text && *text) parsie = new parser(text, 1);
-	else parsie = new parser(this_lang->input_file, 0);
-	root=new unit(cfg->text_level, parsie);
+	if (text && *text) parsie = new parser(text, PARSER_MODE_INPUT);
+	else parsie = new parser(this_lang->input_file, PARSER_MODE_FILE);
+	root = new unit(scfg->_text_level, parsie);
 	delete parsie;
 	return root;
 }
 
-
+#ifdef CLASSIC_BOOLTAB
 
 char *fntab(const char *s, const char *t)
 {
@@ -362,11 +372,16 @@ char *fntab(const char *s, const char *t)
 	int tmp;
 	tab=(char *)xmalloc(256); for(tmp=0; tmp<256; tmp++) tab[tmp] = (unsigned char)tmp;  //identity mapping
 	if(cfg->paranoid && (tmp=strlen(s)-strlen(t)) && t[1]) 
-		shriek(811, fmt(tmp>0 ? "Not enough (%d) resultant elements"
-					: "Too many (%d) resultant elements",abs(tmp)));
+		shriek(811, tmp>0 ? "Not enough (%d) resultant elements"
+				  : "Too many (%d) resultant elements",abs(tmp));
 	if(!t[1]) for (tmp=0; s[tmp]; tmp++) tab[(unsigned char)s[tmp]]=*t;
 	else for (tmp=0;s[tmp]&&t[tmp];tmp++) tab[(unsigned char)(s[tmp])]=t[tmp];
-
+	
+	int ids = 0;
+	DBG(1,for (tmp = 0; tmp < 256; tmp++) if ((unsigned char)tab[tmp] == tmp) ids++;\
+	int globs = t[1] ? 0 : strlen(s);\
+	int rest = 256 - ids - globs;\
+	fprintf(STDDBG, "Adding a function, %d ids, %d being mapped to %c, and %d nontrivials\n", ids, globs, t[1] ? '#' : t[0], rest);)
 	return(tab);
 }
 
@@ -378,21 +393,25 @@ bool *booltab(const char *s)
 	bool mode = true;
 	tab = (bool *)xcalloc(sizeof(bool), 256);         //should always return zero-filled array
 	
-	DEBUG(0,0,fprintf(STDDBG, "gonna booltab out of %s\n", s););
+	D_PRINT(0, "gonna booltab out of %s\n", s);
 	if (*s==EXCLAM) memset(tab, true, 256*sizeof(bool));
 	for(tmp=s; *tmp; tmp++) switch (*tmp) {
 		case EXCLAM: mode = !mode; break;
 		case ESCAPE: if (!*++tmp) tmp--;	// and fall through
 		default:  tab[(unsigned char)(*tmp)]=mode;
 	}
+	DBG(2, int yes = 0; for(int i = 0; i < 256; i++) if (tab[i]) yes++;\
+	fprintf(STDDBG, "Adding a booltab, %d yes, %d no\n", yes, 256 - yes);)
 	return(tab);
 }
 
+#endif
+
 /*
  * be slow and robust
  * if filename is absolute, ignore the dirname
  * if filename begins with "./", ignore the dirname
- * non-absolute dirnames start from cfg->base_dir
+ * non-absolute dirnames start from scfg->base_dir
  * convert any slashes to backslashes in DOS
  * the compiler will notice if '/'==SLASH and test only once
  */
@@ -404,13 +423,14 @@ char *compose_pathname(const char *filename, const char *dirname, const char *tr
 	register int tmp=0;
 	char *pathname;
 
-	if (!filename || !dirname || !treename) return strdup("");
-	if (!*dirname) dirname = ".";
-	pathname = (char *)xmalloc(strlen(filename) + strlen(dirname) + strlen(treename) + strlen(cfg->base_dir) + 4);
+	if (!filename) filename = "";
+	if (!dirname || !*dirname) dirname = ".";
+	if (!treename) treename = "";
+	pathname = (char *)xmalloc(strlen(filename) + strlen(dirname) + strlen(treename) + strlen(scfg->base_dir) + 4);
 	if (IS_NOT_SLASH(*filename) && (filename[0]!='.' || IS_NOT_SLASH(filename[1]))) {
 		if (IS_NOT_SLASH(*dirname)) {
 			if (IS_NOT_SLASH(*treename)) {
-				strcpy(pathname, cfg->base_dir);
+				strcpy(pathname, scfg->base_dir);
 				tmp = strlen(pathname);
 				if (IS_NOT_SLASH(pathname[tmp-1])) pathname[tmp++] = SLASH;
 			}
@@ -442,7 +462,7 @@ char *limit_pathname(const char *filename, const char *dirname)
 {
 	char *p;
 	char *r;
-	if ((int)strlen(filename) >= cfg->scratch) shriek(864, "File name too long");
+	if ((int)strlen(filename) >= scfg->scratch_size) shriek(864, "File name too long");
 	strcpy(scratch, filename);
 	for (p = scratch; !IS_NOT_SLASH(*p); p++);
 	for (r = scratch; (*r = *p); p++, r++) {
@@ -472,7 +492,7 @@ int get_timestamp(char *filename)
 
 file::~file()
 {
-	if (ref_count) shriek(862, fmt("File %s not unclaimed", filename));
+	if (ref_count) shriek(862, "File %s not unclaimed", filename);
 	free(data);
 	free(filename);
 }
@@ -491,17 +511,18 @@ inline bool reclaim(file *ff, const char *flags, const char *description, void o
 	ff->timestamp = ts;
 	f = fopen(ff->filename, flags, description);
 	fseek(f, 0, SEEK_END);
-	ff->data = (char *)xrealloc(ff->data, tmp = ftell(f) + 1);
+	tmp = ftell(f) + 1;
+	ff->data = (char *)xrealloc(ff->data, tmp);
 	fseek(f, 0, SEEK_SET);
 	len = fread(ff->data, 1, tmp, f);
 	if (len < 0) {
 		if (!description) description = "an unspecified stuff";
-		shriek(445, fmt("Failed to read %s from %s", description, ff->filename));
+		shriek(445, "Failed to read %s from %s", description, ff->filename);
 	}
 	ff->data[len] = 0;
 	if (oven != NULL) oven(ff->data, len);
 	fclose(f);
-	DEBUG(1,0,fprintf(STDDBG, "cache update for %s\n", ff->filename);)
+	D_PRINT(1, "cache update for %s\n", ff->filename);
 	return true;
 }
 
@@ -518,13 +539,13 @@ file *claim(const char *filename, const char *dirname, const char *treename, con
 		file_cache->dupkey = file_cache->dupdata = false;
 	}
 
-	if (!filename) shriek(445, fmt("Failed to read %s (no file name)", description ? description : "??"));
+	if (!filename) shriek(445, "Failed to read %s (no file name)", description ? description : "??");
 
 	pathname = compose_pathname(filename, dirname, treename);
 	ff = file_cache->translate(pathname);
 	if (ff) {
 		reclaim(ff, flags, description, oven);
-		DEBUG(1,0,fprintf(STDDBG, "cache hit on %s\n", pathname);)
+		D_PRINT(1, "cache hit on %s\n", pathname);
 		free(pathname);
 		ff->ref_count++;
 		return ff;
@@ -532,14 +553,14 @@ file *claim(const char *filename, const char *dirname, const char *treename, con
 
 	f = fopen(pathname, flags, description);
 	if (!f && !description) return NULL;
-	if (fseek(f, 0, SEEK_END)) tmp = cfg->dev_txtlen;
+	if (fseek(f, 0, SEEK_END)) tmp = cfg->dev_text_len;
 	else tmp = ftell(f) + 1;
 	data = (char *)xmalloc(tmp);
 	fseek(f, 0, SEEK_SET);
 	tmp = fread(data, 1, tmp, f);
 	if (tmp == -1) {
 		if (!description) return NULL;
-		shriek(445, fmt("Failed to read %s from %s", description, pathname));
+		shriek(445, "Failed to read %s from %s", description, pathname);
 	}
 	data[tmp] = 0;	  // remember DOS crlfs. tmp suits here.
 	if (oven != NULL) oven(data, tmp);
@@ -551,7 +572,7 @@ file *claim(const char *filename, const char *dirname, const char *treename, con
 	ff->filename = pathname;
 	ff->timestamp = get_timestamp(pathname);
 	file_cache->add(pathname, ff);
-	DEBUG(1,0,fprintf(STDDBG, "cache miss on %s\n", pathname);)
+	D_PRINT(1, "cache miss on %s\n", pathname);
 	return ff;
 }
 
@@ -562,9 +583,10 @@ void uncache_file(char *, file *ff, void *)	// the first argument is ignored
 
 void unclaim(file *ff)
 {
-	ff->ref_count--;
-	if (!ff->ref_count) {
-		if (cfg->lowmemory) uncache_file(NULL, ff, NULL);
+	if (ff->ref_count <= 0) shriek(461, "Forgot to unclaim()");
+	
+	if (!--ff->ref_count) {
+		if (scfg->memory_low) uncache_file(NULL, ff, NULL);
 	}
 }
 
@@ -587,11 +609,16 @@ static inline void release(char **buffer)
 static inline void compile_rules()
 {
 	int tmp = cfg->default_lang;
-	_next_rule_line = (char *)xmalloc(cfg->max_line+1);
+	_next_rule_line = (char *)xmalloc(scfg->max_line_len+1);
 	for (int i=0; i<cfg->n_langs; i++) {
 		cfg->default_lang = i;
-		if (cfg->langs[i]->n_voices)
+		if (cfg->langs[i]->n_voices) try {
 			cfg->langs[i]->compile_rules();
+		} catch (any_exception *e) {
+			errors++;
+			delete e;
+		}
+		if (errors > 0) shriek(811, "Rules for %s cannot be compiled", cfg->langs[i]->name);
 	}
 	free(_next_rule_line); _next_rule_line = NULL;
 	cfg->default_lang = tmp;
@@ -606,57 +633,66 @@ static inline void compile_rules()
 		Takes nearly as much time as killing and restarting
  *	epos_catharsis(): to release as much as possible, but leave a way back
  */
-
+ 
+void use_async_sputs();
 
 void epos_init()	 //Some global sanity checks made here
 {
 #ifdef HAVE_SYSLOG_H
 	openlog("epos", LOG_CONS, LOG_DAEMON);
 #endif
-
-	if (!cfg->loaded)	cfg->stddbg = stdout,
+	if (!scfg->_loaded)cfg->stddbg = stdout,
 				cfg->stdshriek = stderr;
 	if (sizeof(int)<4*sizeof(char) || sizeof(int *)<4*sizeof(char)) 
-		shriek (862, fmt("You dwarf! I want at least 32 bit arithmetic & pointery [%d]", sizeof(int)));
+		shriek (862, "I need at least 32 bit arithmetic & pointery [%d]", sizeof(int));
 	if (sizeof(int) > sizeof(void *)) shriek(862, "Your integers are longer than pointers!\n"
 				"Turn hash_table::forall() fns in hash.h the other way round.");
 	if ((unsigned char)-1 != 255) shriek(862, "Your chars are not 8-bit? Funny.");
-	if (sizeof(unsigned short int) != 2) shriek(862, "Short ints not short enough");
-	if (*(short int *)"wxyz" == 256 * 'w' + 'x') cfg->big_endian = true;
-	if (*(short int *)"wxyz" != 256 * 'x' + 'w' && !cfg->big_endian) shriek(862,
+	if (*(int16_t *)"wxyz" == 256 * 'w' + 'x') scfg->_big_endian = true;
+	if (*(int16_t *)"wxyz" != 256 * 'x' + 'w' && !scfg->_big_endian) shriek(862,
 				"Not little-endian nor big-endian. Whew!");
 
 #ifdef HAVE_TIME_H
 	srand(time(NULL));	// randomize
 #endif
-
-	if (!esctab) esctab = FOREVER(fntab(cfg->token_esc, cfg->value_esc));
+	load_default_charset();
+	if (!esctab) esctab = new charxlat(scfg->_token_esc, scfg->_value_esc, false);
 
 	config_init();
 
 #ifdef DEBUGGING
-	if (cfg->use_dbg && cfg->stddbg_file && *cfg->stddbg_file)
-		cfg->stddbg = fopen(cfg->stddbg_file,"w","debugging messages");
+	if (scfg->debug && scfg->stddbg_file && *scfg->stddbg_file)
+		cfg->stddbg = fopen(scfg->stddbg_file,"w","debugging messages");
 #else
-	if (cfg->use_dbg) shriek(813, "Either disable debugging in config file, or #define it in interf.h");
+	if (scfg->debug) shriek(813, "Either disable debugging in config file, or #define it in interf.h");
 #endif
 	cfg->stdshriek = stderr;
-	if (cfg->stdshriek_file && *cfg->stdshriek_file)
-		cfg->stdshriek = fopen(cfg->stdshriek_file, "w", "error messages");
+	if (scfg->stdshriek_file && *scfg->stdshriek_file)
+		cfg->stdshriek = fopen(scfg->stdshriek_file, "w", "error messages");
 		
 //	if (!_subst_buff) _subst_buff = (char *)xmalloc(MAX_GATHER+2);
 //	if (!_gather_buff) _gather_buff = (char *)xmalloc(MAX_GATHER+2);
-	if (!_resolve_vars_buff) _resolve_vars_buff = (char *)xmalloc(cfg->max_line+1); 
-	if (!scratch) scratch = (char *)xmalloc(cfg->scratch+1);
+	if (!_resolve_vars_buff) _resolve_vars_buff = (char *)xmalloc(scfg->max_line_len+1); 
+	if (!scratch) {
+		scratch = (char *)xmalloc(scfg->scratch_size+1);
+		scratch[scfg->scratch_size] = 0;
+	}
+	use_async_sputs();
 	
 	compile_rules();
+	update_sampa();
 
-	DEBUG(1,10,fprintf(STDDBG,"struct configuration is %d bytes\n", (int)sizeof(configuration));)
-	DEBUG(1,10,fprintf(STDDBG,"struct lang is %d bytes\n", (int)sizeof(lang));)
-	DEBUG(1,10,fprintf(STDDBG,"struct voice is %d bytes\n", (int)sizeof(voice));)
+	D_PRINT(1, "struct unit is %d bytes\n", (int)sizeof(unit));
+	D_PRINT(1, "struct static_configuration is %d bytes\n", (int)sizeof(static_configuration));
+	D_PRINT(1, "struct configuration is %d bytes\n", (int)sizeof(configuration));
+	D_PRINT(1, "struct lang is %d bytes\n", (int)sizeof(lang));
+	D_PRINT(1, "struct voice is %d bytes\n", (int)sizeof(voice));
+	D_PRINT(2, "allocated %d chars, %d chars free\n", get_count_allocated(), 256 - get_count_allocated());
+	D_PRINT(1, "charsets already in use are %s\n", charset_list);
 }
 
 void end_of_eternity();
+void shutdown_sched_aq();
 
 void epos_catharsis()
 {
@@ -666,41 +702,52 @@ void epos_catharsis()
 		_unit_just_unlinked=NULL;
 	}
 
-//	if (_directive_prefices) delete _directive_prefices;
-//	_directive_prefices = NULL;
-
 	// one a_protocol may be lost here, see agent.cc
 
-	cfg->shutdown();
+	cow_catharsis();
+
+	release_sampa();
+
+	cfg->shutdown();	//...FIXME: might need proto_cfg->shutdown; othrws shutdown never called
+	scfg->shutdown();
+	free_extra_options();
+	delete esctab; esctab = NULL;
 
 	shutdown_file_cache();
 	
+	shutdown_enc();
 #ifdef HAVE_SYSLOG_H
 	closelog();
 #endif
 }
 
+void free_replier_table();
+void shutdown_parser();
+void shutdown_partbuffs();
+
 void epos_done()
 {
 	epos_catharsis();
+	free_replier_table();
 	config_release();
 	unit::done();
 	shutdown_hashing();
-	shutdown_langs();
 	shutdown_cfgs();
+	shutdown_sched_aq();
+	shutdown_parser();
 
-//	release(&_subst_buff);
-//	release(&_gather_buff);
 	release(&_resolve_vars_buff);
 	release(&scratch);
+	shutdown_partbuffs();
 	
 	END_OF_ETERNITY;
+	cow_catharsis();
 }
 
 void epos_reinit()
 {
 	epos_catharsis();
-	load_config("default.ini");
+	reinitialize_configuration();
 	epos_init();
 }
 
@@ -722,39 +769,11 @@ bool privileged_exec()
 
 char *current_debug_tag = NULL;
 
-int  debug_config(int area)
-{
-	switch (area) {
-		case _INTERF_: return cfg->interf_dbg;
-		case _RULES_:  return cfg->rules_dbg;
-		case _ELEM_:   return cfg->elem_dbg;
-		case _SUBST_:  return cfg->subst_dbg;
-		case _ASSIM_:  return cfg->assim_dbg;
-		case _SPLIT_:  return cfg->split_dbg;
-		case _PARSER_: return cfg->parser_dbg;
-		case _SYNTH_:  return cfg->synth_dbg;
-		case _CFG_:    return cfg->cfg_dbg;
-		case _DAEMON_: return cfg->daemon_dbg;
-	}
-	shriek(861, fmt("Unknown debug area %d", area));
-	return 0;   // keep the compiler happy
-}
-
-bool debug_wanted(int lev, /*_DEBUG_AREA_*/ int area) 
-{
-	if (!cfg->use_dbg) return false;
-	if (cfg->always_dbg > 10 || cfg->always_dbg < 0) shriek(862, "cfg bogus"); //FIXME: hack
-	if (lev >= cfg->always_dbg) return true;
-	if (area == cfg->focus_dbg) return lev >= debug_config(area);
-	if (lev < cfg->limit_dbg)   return false;
-	return  lev >= debug_config(area);
-}
-
 void debug_prefix(int lev, int area)
 {
 	unuse(lev); unuse(area);
+	color(STDDBG, scfg->normal_color);
 	if (current_debug_tag) fprintf(STDDBG, "[%s] ", current_debug_tag);
-//	if (this_lang && this_lang->name) printf("%s ", this_lang->name);
 }
 
 #endif   // ifdef DEBUGGING
@@ -777,9 +796,9 @@ char *forever(void *heapptr)
 
 void end_of_eternity()
 {
-	DEBUG(3,0,fprintf(STDDBG,"Freeing %d permanent heap buffers.\n", forever_count);)
+	D_PRINT(3, "Freeing %d permanent heap buffers.\n", forever_count);
 	while (forever_count-- > 0) {
-		DEBUG(0,0,fprintf(STDDBG, "pointer number %d was %p\n", forever_count, forever_ptr_list[forever_count]);)
+		D_PRINT(0, "pointer number %d was %p\n", forever_count, forever_ptr_list[forever_count]);
 		free(forever_ptr_list[forever_count]);
 	}
 }
@@ -809,15 +828,6 @@ void call_abort()
 #endif
 }
 
-//#ifndef HAVE_STRDUP
-
-char *strdup(const char*src)
-{
-	return strcpy((char *)xmalloc(strlen(src)+1), src);
-}
-
-//#endif   // ifdef HAVE_STRDUP
-
 #ifndef HAVE_TERMINATE
 
 void terminate(void)
@@ -835,3 +845,4 @@ int fork()
 #endif   // ifdef HAVE_FORK
 
 
+
