@@ -345,7 +345,7 @@ unit::insert(UNIT target, bool backwards, char what, charclass *left, charclass
 	unit *tmpu;
 
 	if (depth == target) {
-		D_PRINT(1, "inner unit::insert %c %c %c\n",Prev(depth)->inside(), cont, Next(depth)->inside());
+		D_PRINT(1, "inner unit::insert %c %c %c\n",Prev(depth)->inside_or_zero(), cont, Next(depth)->inside_or_zero());
 		D_PRINT(1, "   env is %c %c\n",
 				left->ismember(Prev(depth)->inside())+'0',
 				right->ismember(Next(depth)->inside())+'0');
@@ -1029,7 +1029,7 @@ unit::analyze(UNIT target, hash *table, int unanal_unit_penalty, int unanal_part
 
 	int l;
 	char *b = gather(false, false);
-	l = gbsize;
+	l = gblen;
 	if (++l > vb_size) {
 		vb_size = l;
 		D_PRINT(1, "Growing Viterbi buffer to %d items\n", vb_size);
@@ -1046,13 +1046,16 @@ unit::analyze(UNIT target, hash *table, int unanal_unit_penalty, int unanal_part
 			char *r = j >= i - table->longest ? table->translate(b + j) : 0;
 			int badness = r ? atoi(r)
 				: (i - j) * unanal_unit_penalty + unanal_part_penalty;
+			D_PRINT(0, "Having a string from %d to %d would bring badness %d + %d\n", j, i, vb[j].badness, badness);
 			badness += vb[j].badness;
 			if (badness < vb[i].badness) {
+				D_PRINT(0, "Found an improvement over %d at position %d\n", vb[i].badness, i);
 				vb[i].badness = badness;
 				vb[i].last_len = i - j;
 			}
 		}
 		b[i] = tmp;
+		D_PRINT(0, "Optimum way for arriving at position %d is to start at %d and get %d\n", i, i - vb[i].last_len, vb[i].badness);
 	}
 
 	int n = 0;	
@@ -1410,13 +1413,13 @@ unit::segs(UNIT target, hash *dinven)
 		_d_descr[3]=0;
 		_d_descr[2]=QUESTION_MARK;
 		_d_descr[1]=inside();
-		_d_descr[0]=Prev(depth)->inside();
+		_d_descr[0]=Prev(depth)->inside_or_zero();
 		seg (dinven);
 		_d_descr[0]=QUESTION_MARK;
 		seg (dinven);
-		_d_descr[2]=Next(depth)->inside();
+		_d_descr[2]=Next(depth)->inside_or_zero();
 		seg (dinven);
-		_d_descr[0]=Prev(depth)->inside();
+		_d_descr[0]=Prev(depth)->inside_or_zero();
 		seg (dinven);
 		D_PRINT(1, "unit::segs is done in: %c\n", cont);
 	}
