@@ -48,7 +48,8 @@ class unit
 	unit *firstborn, *lastborn;       //layer lower by one
 	unit *father;                     //layer greater by one
 	UNIT depth;        // 0=segment 1=phone 2=syllable 3=word...
-	int  cont;         // content (or terminating character)
+	bool scope;                       //true=don't pass on Next/Prev requests
+	short int  cont;         // content (or terminating character)
 	void insert_begin(unit *member, unit*to);         //insert a train of units
 	void insert_end(unit *from, unit *member);        //insert as the last member
 				      //if NULL, shrieks unintuitively
@@ -75,7 +76,6 @@ class unit
 	float t;
 	marker *m;
 //  public:
-	bool scope;                       //true=don't pass on Next/Prev requests
 		unit(UNIT layer, parser *);
 		unit(UNIT layer, int content); 
 		unit();               //(empty unit) constructor
@@ -130,6 +130,7 @@ class unit
 				      // userfn ^ whether applied
 	int effective(FIT_IDX which_quantity);  //evaluate total F, I or T
 	inline unsigned char inside() { return (unsigned char)cont; };
+	inline unsigned char inside_or_zero() { return cont == ABSENT_CHARACTER ? LITERAL_ZERO : inside(); };
 	unit *ancestor(UNIT level);   // the unit (depth level) wherein this lies
 	int  index(UNIT what, UNIT where);
 	int  count(UNIT what);
